@@ -1,16 +1,7 @@
-from collections import deque
 from collections.abc import Mapping, MutableMapping, Sequence
 from urllib.parse import urlsplit
 import itertools
-import json
 import re
-import sys
-
-# The files() API was added in Python 3.9.
-if sys.version_info >= (3, 9):  # pragma: no cover
-    from importlib import resources
-else:  # pragma: no cover
-    import importlib_resources as resources  # type: ignore
 
 
 class URIDict(MutableMapping):
@@ -37,32 +28,22 @@ class URIDict(MutableMapping):
     def __iter__(self):
         return iter(self.store)
 
-    def __len__(self):
+    def __len__(self):  # pragma: no cover -- untested, but to be removed
         return len(self.store)
 
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover -- untested, but to be removed
         return repr(self.store)
 
 
-class Unset(object):
+class Unset:
     """
     An as-of-yet unset attribute or unprovided default parameter.
     """
 
-    def __repr__(self):
+    def __repr__(self):  # pragma: no cover
         return "<unset>"
 
 
-def load_schema(name):
-    """
-    Load a schema from ./schemas/``name``.json and return it.
-    """
-
-    path = resources.files(__package__).joinpath(f"schemas/{name}.json")
-    data = path.read_text(encoding="utf-8")
-    return json.loads(data)
-
-
 def format_as_index(container, indices):
     """
     Construct a single string containing indexing operations for the indices.
@@ -109,11 +90,8 @@ def extras_msg(extras):
     Create an error message for extra items or properties.
     """
 
-    if len(extras) == 1:
-        verb = "was"
-    else:
-        verb = "were"
-    return ", ".join(repr(extra) for extra in sorted(extras)), verb
+    verb = "was" if len(extras) == 1 else "were"
+    return ", ".join(repr(extra) for extra in extras), verb
 
 
 def ensure_list(thing):
@@ -219,15 +197,33 @@ def find_evaluated_item_indexes_by_schema(validator, instance, schema):
     if "items" in schema:
         return list(range(0, len(instance)))
 
-    if "$ref" in schema:
-        scope, resolved = validator.resolver.resolve(schema["$ref"])
-        validator.resolver.push_scope(scope)
+    ref = schema.get("$ref")
+    if ref is not None:
+        resolved = validator._resolver.lookup(ref)
+        evaluated_indexes.extend(
+            find_evaluated_item_indexes_by_schema(
+                validator.evolve(
+                    schema=resolved.contents,
+                    _resolver=resolved.resolver,
+                ),
+                instance,
+                resolved.contents,
+            ),
+        )
 
-        try:
-            evaluated_indexes += find_evaluated_item_indexes_by_schema(
-                validator, instance, resolved)
-        finally:
-            validator.resolver.pop_scope()
+    dynamicRef = schema.get("$dynamicRef")
+    if dynamicRef is not None:
+        resolved = validator._resolver.lookup(dynamicRef)
+        evaluated_indexes.extend(
+            find_evaluated_item_indexes_by_schema(
+                validator.evolve(
+                    schema=resolved.contents,
+                    _resolver=resolved.resolver,
+                ),
+                instance,
+                resolved.contents,
+            ),
+        )
 
     if "prefixItems" in schema:
         evaluated_indexes += list(range(0, len(schema["prefixItems"])))
@@ -256,8 +252,8 @@ def find_evaluated_item_indexes_by_schema(validator, instance, schema):
     for keyword in ["allOf", "oneOf", "anyOf"]:
         if keyword in schema:
             for subschema in schema[keyword]:
-                errs = list(validator.descend(instance, subschema))
-                if not errs:
+                errs = next(validator.descend(instance, subschema), None)
+                if errs is None:
                     evaluated_indexes += find_evaluated_item_indexes_by_schema(
                         validator, instance, subschema,
                     )
@@ -277,41 +273,51 @@ def find_evaluated_property_keys_by_schema(validator, instance, schema):
         return []
     evaluated_keys = []
 
-    if "$ref" in schema:
-        scope, resolved = validator.resolver.resolve(schema["$ref"])
-        validator.resolver.push_scope(scope)
+    ref = schema.get("$ref")
+    if ref is not None:
+        resolved = validator._resolver.lookup(ref)
+        evaluated_keys.extend(
+            find_evaluated_property_keys_by_schema(
+                validator.evolve(
+                    schema=resolved.contents,
+                    _resolver=resolved.resolver,
+                ),
+                instance,
+                resolved.contents,
+            ),
+        )
 
-        try:
-            evaluated_keys += find_evaluated_property_keys_by_schema(
-                validator, instance, resolved,
-            )
-        finally:
-            validator.resolver.pop_scope()
+    dynamicRef = schema.get("$dynamicRef")
+    if dynamicRef is not None:
+        resolved = validator._resolver.lookup(dynamicRef)
+        evaluated_keys.extend(
+            find_evaluated_property_keys_by_schema(
+                validator.evolve(
+                    schema=resolved.contents,
+                    _resolver=resolved.resolver,
+                ),
+                instance,
+                resolved.contents,
+            ),
+        )
 
     for keyword in [
         "properties", "additionalProperties", "unevaluatedProperties",
     ]:
         if keyword in schema:
-            if validator.is_type(schema[keyword], "boolean"):
-                for property, value in instance.items():
-                    if validator.evolve(schema=schema[keyword]).is_valid(
-                        {property: value},
-                    ):
-                        evaluated_keys.append(property)
+            schema_value = schema[keyword]
+            if validator.is_type(schema_value, "boolean") and schema_value:
+                evaluated_keys += instance.keys()
 
-            if validator.is_type(schema[keyword], "object"):
-                for property, subschema in schema[keyword].items():
-                    if property in instance and validator.evolve(
-                        schema=subschema,
-                    ).is_valid(instance[property]):
+            elif validator.is_type(schema_value, "object"):
+                for property in schema_value:
+                    if property in instance:
                         evaluated_keys.append(property)
 
     if "patternProperties" in schema:
-        for property, value in instance.items():
-            for pattern, _ in schema["patternProperties"].items():
-                if re.search(pattern, property) and validator.evolve(
-                    schema=schema["patternProperties"],
-                ).is_valid({property: value}):
+        for property in instance:
+            for pattern in schema["patternProperties"]:
+                if re.search(pattern, property):
                     evaluated_keys.append(property)
 
     if "dependentSchemas" in schema:
@@ -325,8 +331,8 @@ def find_evaluated_property_keys_by_schema(validator, instance, schema):
     for keyword in ["allOf", "oneOf", "anyOf"]:
         if keyword in schema:
             for subschema in schema[keyword]:
-                errs = list(validator.descend(instance, subschema))
-                if not errs:
+                errs = next(validator.descend(instance, subschema), None)
+                if errs is None:
                     evaluated_keys += find_evaluated_property_keys_by_schema(
                         validator, instance, subschema,
                     )
@@ -347,85 +353,3 @@ def find_evaluated_property_keys_by_schema(validator, instance, schema):
                 )
 
     return evaluated_keys
-
-
-def _schema_is_referenced(schema, parent_schema):
-    """
-    Checks if a schema is referenced by another schema
-    """
-    return (
-        "$id" in schema
-        and "$ref" in parent_schema
-        and parent_schema["$ref"] == schema["$id"]
-    )
-
-
-def _find_dynamic_anchor_extender(validator, scopes, fragment, schema):
-    """
-    Find a schema that extends the dynamic anchor
-    """
-    for url in scopes:
-        with validator.resolver.resolving(url) as parent_schema:
-            if _schema_is_referenced(schema, parent_schema):
-                return validator.resolver.resolve_fragment(
-                    parent_schema,
-                    fragment,
-                )
-
-
-def _find_dynamic_anchor_intermediate(validator, scopes, fragment):
-    """
-    Find a schema that extends the dynamic anchor by an intermediate schema
-    """
-    for url in scopes:
-        with validator.resolver.resolving(url) as schema:
-            if "$id" in schema:
-                for intermediate_url in scopes:
-                    with validator.resolver.resolving(
-                            intermediate_url) as intermediate_schema:
-                        for subschema in search_schema(
-                                intermediate_schema, match_keyword("$ref")):
-                            if _schema_is_referenced(subschema, schema):
-                                return _find_dynamic_anchor_extender(
-                                    validator,
-                                    scopes,
-                                    fragment,
-                                    subschema,
-                                )
-
-
-def dynamic_anchor_extender(validator, scopes, fragment, schema, subschema):
-    extender_schema = _find_dynamic_anchor_extender(
-        validator, scopes, fragment, schema,
-    )
-    if not extender_schema:
-        extender_schema = _find_dynamic_anchor_extender(
-            validator, scopes, fragment, subschema,
-        )
-    if not extender_schema:
-        extender_schema = _find_dynamic_anchor_intermediate(
-            validator, scopes, fragment,
-        )
-
-    return extender_schema
-
-
-def match_keyword(keyword):
-    def matcher(value):
-        if keyword in value:
-            yield value
-    return matcher
-
-
-def search_schema(schema, matcher):
-    """Breadth-first search routine."""
-    values = deque([schema])
-    while values:
-        value = values.pop()
-        if isinstance(value, list):
-            values.extendleft(value)
-            continue
-        if not isinstance(value, dict):
-            continue
-        yield from matcher(value)
-        values.extendleft(value.values())
